@@ -3,12 +3,14 @@
 
 require 'mkmf'
 
+$CFLAGS += ' -Wall '
+
 if /linux/ =~ RUBY_PLATFORM
    $LDFLAGS += '-Wl,--rpath -Wl,$(EPREFIX)/lib'
 elsif /solaris/ =~ RUBY_PLATFORM
    $LDFLAGS += '-R$(EPREFIX)/lib'
 elsif /hpux/ =~ RUBY_PLATFORM
-   $LDFLAGS += '+b$(EPREFIX)/lib';
+   $LDFLAGS += '+b$(EPREFIX)/lib'
 elsif /aix/ =~ RUBY_PLATFORM
    $LDFLAGS += '-Wl,-blibpath:$(EPREFIX)/lib'
 end
