@@ -1,11 +1,11 @@
 /*****************************************************************************
- * RRDtool 1.3.9  Copyright by Tobi Oetiker, 1997-2009
+ * RRDtool 1.4.2  Copyright by Tobi Oetiker, 1997-2009
  * This code is stolen from rateup (mrtg-2.x) by Dave Rand
  *****************************************************************************
  * diff calculate the difference between two very long integers available as
  *      strings
  *****************************************************************************
- * $Id: rrd_diff.c 1946 2009-10-24 10:46:42Z oetiker $
+ * $Id: rrd_diff.c 1970 2009-11-15 11:54:23Z oetiker $
  * $Log$
  * Revision 1.4  2003/03/10 00:30:34  oetiker
  * handle cases with two negative numbers
@@ -35,11 +35,8 @@
  *
  *****************************************************************************/
 
-#include "rrd_tool.h"
-
-#ifdef WIN32
 #include <ctype.h>
-#endif
+#include "rrd_tool.h"
 
 double rrd_diff(
     char *a,
